/*YAAME - Amidar driver
  Also supports Turtles

  CPU : Z80 (3.57 mhz)
  CPU : Z80 (1.78 mhz)
  Sound : 2xAY whatever (1.78 mhz)
  Graphics : 256x256, 32 colours

  Known problems :

  Background colour should be blue on Turtles
  Too fast?
*/

#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "yaame.h"
#include "ay.h"

int amidarint=0;
unsigned char ram2[65536];
struct mz80context z802;
int amidarcount=0;

int amidartimer[20] = {
0x00, 0x10, 0x00, 0x10, 0x20, 0x30, 0x20, 0x30, 0x40, 0x50,
0x80, 0x90, 0x80, 0x90, 0xa0, 0xb0, 0xa0, 0xb0, 0xc0, 0xd0
};

UINT8 amidartimerread()
{
      amidarcount++;
      if (amidarcount==20)
         amidarcount=0;
      return amidartimer[amidarcount];
}

static unsigned char amidarcolour[] =
{
	/* palette */
	0x00,0x07,0xC0,0xB6,
	0x00,0x38,0xC5,0x67,
	0x00,0x30,0x07,0x3F,
	0x00,0x07,0x30,0x3F,
	0x00,0x3F,0x30,0x07,
	0x00,0x38,0x67,0x3F,
	0x00,0xFF,0x07,0xDF,
	0x00,0xF8,0x07,0xFF
};

static unsigned char turtlescolour[] =
{
	/* palette */
	0x00,0xC0,0x57,0xFF,
	0x00,0x66,0xF2,0xFE,
	0x00,0x2D,0x12,0xBF,
	0x00,0x2F,0x7D,0xB8,
	0x00,0x72,0xD2,0x06,
	0x00,0x94,0xFF,0xE8,
	0x00,0x54,0x2F,0xF6,
	0x00,0x24,0xBF,0xC6
};

void amidarirq(UINT32 addr, UINT8 data, struct MemoryWriteByte *m)
{
      amidarint=1;
}

UINT8 amidarreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_SPACE])
           temp&=~8;
        if (keys[KEY_RIGHT])
           temp&=~0x10;
        if (keys[KEY_LEFT])
           temp&=~0x20;
        if (keys[KEY_C])
           temp&=~0x80;
        return temp;
}

UINT8 amidarreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_1])
           temp&=~0x80;
        return temp;
}

UINT8 amidarreadin2(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xF1;
        if (keys[KEY_UP])
           temp&=~0x10;
        if (keys[KEY_DOWN])
           temp&=~0x40;
        return temp;
}

UINT8 amidarreaddsw(UINT32 addr, struct MemoryReadByte *m)
{
        return 0xFF;
}

struct MemoryReadByte amidarread[] =
{
        {0xB000,0xB000,amidarreadin0},
        {0xB010,0xB010,amidarreadin1},
        {0xB020,0xB020,amidarreadin2},
        {0xB820,0xB820,amidarreaddsw},
        {-1,-1,NULL}
};

struct MemoryWriteByte amidarwrite[] =
{
        {0,0x3FFF,writerom},
        {0xA008,0xA008,nmienable},
        {0xB800,0xB800,aywriteporta2},
        {0xB810,0xB810,amidarirq},
        {-1,-1,NULL}
};

struct z80PortRead amidarportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite amidarportwrite[] =
{
	{-1,	-1,	NULL}
};

struct MemoryReadByte amidarsoundread[] =
{
        {-1,-1,NULL}
};

struct MemoryWriteByte amidarsoundwrite[] =
{
        {0,0x1FFF,writerom},
        {-1,-1,NULL}
};

struct z80PortRead amidarsoundportread[] =
{
        {0x20,0x20,ayread1p},
        {0x80,0x80,ayread2p},
        {-1,-1,NULL}
};

struct z80PortWrite amidarsoundportwrite[] =
{
        {0x10,0x10,aywritectrl1p},
        {0x20,0x20,aywrite1p},
        {0x40,0x40,aywritectrl2p},
        {0x80,0x80,aywrite2p},
        {-1,-1,NULL}
};

int amidarloadgfx()
{
        int c;
        int bit0,bit1,bit2;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read2planeflipped(0,256,"amidar.5f","amidar.5h");
	     for (c=0;c<32;c++)
	     {
      		   bit0 = (amidarcolour[c] >> 0) & 0x01;
		         bit1 = (amidarcolour[c] >> 1) & 0x01;
		         bit2 = (amidarcolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (amidarcolour[c] >> 3) & 0x01;
		         bit1 = (amidarcolour[c] >> 4) & 0x01;
		         bit2 = (amidarcolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (amidarcolour[c] >> 6) & 0x01;
		         bit2 = (amidarcolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int turtlesloadgfx()
{
        int c;
        int bit0,bit1,bit2;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read2planeflipped(0,256,"turt_vid.5h","turt_vid.5f");
	     for (c=0;c<32;c++)
	     {
      		   bit0 = (turtlescolour[c] >> 0) & 0x01;
		         bit1 = (turtlescolour[c] >> 1) & 0x01;
		         bit2 = (turtlescolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (turtlescolour[c] >> 3) & 0x01;
		         bit1 = (turtlescolour[c] >> 4) & 0x01;
		         bit2 = (turtlescolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (turtlescolour[c] >> 6) & 0x01;
		         bit2 = (turtlescolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int amidarinit()
{
        printf("Loading amidar ROMs\n");
        loadrom("amidar.2c",0,4096);
        loadrom("amidar.2e",0x1000,4096);
        loadrom("amidar.2f",0x2000,4096);
        loadrom("amidar.2h",0x3000,4096);
        loadrom2("amidar.5c",0,4096);
        loadrom2("amidar.5d",0x1000,4096);
        amidarloadgfx();
        initz802(amidarsoundread,amidarsoundwrite,amidarsoundportread,amidarsoundportwrite);
        initz80(amidarread,amidarwrite,amidarportread,amidarportwrite);
        printf("amidar loaded\n");
        nmi=0;
        initay(2);
        ayportb2=amidartimerread;
        ayclock1=ayclock2=1789750;
        return 0;
}

int turtlesinit()
{
        printf("Loading Turtles ROMs\n");
        loadrom("turt_vid.2c",0,4096);
        loadrom("turt_vid.2e",0x1000,4096);
        loadrom("turt_vid.2f",0x2000,4096);
        loadrom("turt_vid.2h",0x3000,4096);
        loadrom("turt_vid.2j",0x4000,4096);
        loadrom2("turt_snd.5c",0,4096);
        loadrom2("turt_snd.5d",0x1000,4096);
        turtlesloadgfx();
        initz802(amidarsoundread,amidarsoundwrite,amidarsoundportread,amidarsoundportwrite);
        initz80(amidarread,amidarwrite,amidarportread,amidarportwrite);
        printf("Turtles loaded\n");
        nmi=0;
        initay(2);
        ayportb2=amidartimerread;
        ayclock1=ayclock2=1789750;
        return 0;
}

void amidardraw()
{
        int x,y,colset,c,tile;
        int pos=0x9000,pos2=0x9800;
        for (x=0;x<32;x++)
        {
                pos2=0x9800;
                for (y=0;y<32;y++)
                {
                        pos2++;
                        colset=ram[pos2++];
                        drawtilenew(ram[pos++],(31-x)*8,y*8,colset,0,0,2);
                }
        }
        for (c=0x9840;c<0x985F;c+=4)
        {
                y=ram[c+3]+1;
                x=ram[c];
                if (x>1&&y>1)
                {
                        tile=ram[c+1]&0x3F;
                        tile<<=2;
                        colset=ram[c+2]&7;
                        drawtilemasknew(tile,x+8,y,colset,0,0,2);
                        drawtilemasknew(tile+2,x,y,colset,0,0,2);
                        drawtilemasknew(tile+1,x+8,y+8,colset,0,0,2);
                        drawtilemasknew(tile+3,x,y+8,colset,0,0,2);
                }
        }
        waitforsync();
        blit(buffer,screen,0,0,0,0,256,256);
}

void amidarrun()
{
        int c;
        while (!keys[KEY_ESC])
        {
                for (c=0;c<10;c++)
                {
                        mz80SetContext(&z80);
                        if (mz80exec(5120)!=0x80000000)
                        {
                              printf("It crashed\n");
                              exit(-1);
                        }
                        mz80GetContext(&z80);
                        mz80SetContext(&z802);
                        if (amidarint)
                        {
                              mz80int(0);
                              amidarint=0;
                        }
                        if (mz80exec(2560)!=0x80000000)
                        {
                              printf("It crashed\n");
                              exit(-1);
                        }
                        mz80GetContext(&z802);
                }
                mz80SetContext(&z80);
                amidardraw();
                if (nmi)
                   mz80nmi();
                mz80GetContext(&z80);
        }
}
