/*YAAME - Anteater driver
  Also supports Super Cobra

  CPU : Z80 (3.57 mhz)
  CPU : Z80 (1.78 mhz) (not emulated)
  Sound : 2xAY whatever (1.78 mhz) (not emulated)
  Graphics : 224x256, 32 colours

  Known problems :

  Some bad colours in Anteater
  You can't shoot in Super Cobra for some reason (although you can drop missiles)
*/

#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "yaame.h"

int scobra=0;

UINT8 anteaterreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_C])
           temp&=~0x80;
        if (keys[KEY_LEFT])
           temp&=~0x20;
        if (keys[KEY_RIGHT])
           temp&=~0x10;
        if (keys[KEY_DOWN])
           temp&=~0x8;
        if (keys[KEY_UP])
           temp&=~0x4;
        if (keys[KEY_SPACE])
           temp&=~0x2;
        return temp;
}

UINT8 anteaterreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        return 0xFF;
}

UINT8 anteaterreadin2(UINT32 addr, struct MemoryReadByte *m)
{
        if (keys[KEY_1])
           return 0xF6;
        return 0xF7;
}

struct MemoryReadByte anteaterread[] =
{
        {0x9800,0x9800,anteaterreadin0},
        {0x9801,0x9801,anteaterreadin1},
        {0x9802,0x9802,anteaterreadin2},
        {-1,-1,NULL}
};

UINT8 scobrareadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_LEFT])
           temp&=~0x20;
        if (keys[KEY_RIGHT])
           temp&=~0x10;
        if (keys[KEY_SPACE])
           temp&=~0x8;
        if (keys[KEY_C])
           temp&=~0x4;
        if (keys[KEY_ALT])
           temp&=~0x2;
        return temp;
}

UINT8 scobrareadin1(UINT32 addr, struct MemoryReadByte *m)
{
        if (keys[KEY_1])
           return 0x7F;
        return 0xFF;
}


UINT8 scobrareadin2(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_DOWN])
           temp&=~0x40;
        if (keys[KEY_UP])
           temp&=~0x10;
        return temp;
}

struct MemoryReadByte scobraread[] =
{
        {0x9800,0x9800,scobrareadin0},
        {0x9801,0x9801,scobrareadin1},
        {0x9802,0x9802,scobrareadin2},
        {-1,-1,NULL}
};

struct MemoryWriteByte anteaterwrite[] =
{
        {0,0x4000,writerom},
        {0xA801,0xA801,nmienable},
        {-1,-1,NULL}
};

struct z80PortRead anteaterportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite anteaterportwrite[] =
{
	{-1,	-1,	NULL}
};

static unsigned char anteatercolour[] =
{
	0x00,0xF6,0x07,0xF0,0x00,0x80,0x3F,0xC7,0x00,0xFF,0x07,0x27,0x00,0xFF,0xC9,0x39,
	0x00,0x3C,0x07,0xF0,0x00,0x27,0x29,0xFF,0x00,0xC7,0x17,0xF6,0x00,0xC7,0x39,0x3F
};

static unsigned char scobracolour[] =
{
	0x00,0xF6,0x07,0xF0,0x00,0x80,0x3F,0xC7,0x00,0xFF,0x07,0x27,0x00,0xFF,0xC9,0x39,
	0x00,0x3C,0x07,0xF0,0x00,0x27,0x29,0xFF,0x00,0xC7,0x17,0xF6,0x00,0xC7,0x39,0x3F
};

int anteaterloadgfx()
{
        int c;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read2planeflipped(0,256,"ra6-5f2","ra6-5h2");
	     for (c=0;c<32;c++)
        {
		         int bit0,bit1,bit2;
		         bit0 = (anteatercolour[c] >> 0) & 0x01;
		         bit1 = (anteatercolour[c] >> 1) & 0x01;
		         bit2 = (anteatercolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (anteatercolour[c] >> 3) & 0x01;
		         bit1 = (anteatercolour[c] >> 4) & 0x01;
		         bit2 = (anteatercolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (anteatercolour[c] >> 6) & 0x01;
		         bit2 = (anteatercolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int anteaterinit()
{
        printf("Loading Anteater ROMs\n");
        loadrom("ra1-2c",0,4096);
        loadrom("ra1-2e",0x1000,4096);
        loadrom("ra1-2f",0x2000,4096);
        loadrom("ra1-2h",0x3000,4096);
        anteaterloadgfx();
        initz80(anteaterread,anteaterwrite,anteaterportread,anteaterportwrite);
        printf("Anteater loaded\n");
        nmi=0;
        return 0;
}

int scobraloadgfx()
{
        FILE *f;
        int c;
        int bit0,bit1,bit2;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read2planeflipped(0,256,"5f","5h");
        f=fopen("82s123.6e","rb");
        if (f)
        {
                fread(anteatercolour,32,1,f);
                fclose(f);
        }
	     for (c=0;c<32;c++)
        {
		         bit0 = (scobracolour[c] >> 0) & 0x01;
		         bit1 = (scobracolour[c] >> 1) & 0x01;
		         bit2 = (scobracolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (scobracolour[c] >> 3) & 0x01;
		         bit1 = (scobracolour[c] >> 4) & 0x01;
		         bit2 = (scobracolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (scobracolour[c] >> 6) & 0x01;
		         bit2 = (scobracolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int scobrainit()
{
        printf("Loading Super Cobra ROMs\n");
        loadrom("2c",0,4096);
        loadrom("2e",0x1000,4096);
        loadrom("2f",0x2000,4096);
        loadrom("2h",0x3000,4096);
        loadrom("2j",0x4000,4096);
        loadrom("2l",0x5000,4096);
        scobraloadgfx();
        initz80(scobraread,anteaterwrite,anteaterportread,anteaterportwrite);
        printf("Super Cobra loaded\n");
        nmi=0;
        scobra=1;
        return 0;
}

void anteaterdrawsprites()
{
        int cc,ccc;
        int spritex,spritey,spritecode,xflip,yflip,spritecol;
        for (cc=15;cc>-1;cc--)
        {
                ccc=cc<<2;
                spritex=ram[0x9040+ccc];
                if (spritex>8 && spritex<0xF8)
                {
                        spritey=ram[0x9043+ccc]+1;
                        spritecol=ram[0x9042+ccc]&7;
                        spritecode=ram[0x9041+ccc]&0x3F;
                        spritecode<<=2;
                        xflip=ram[0x9041+ccc]&0x80;
                        yflip=ram[0x9041+ccc]&0x40;
                        if (!xflip)
                        {
                                if (!yflip)
                                {
                                        drawtilemasknew(spritecode+2,spritex,spritey,spritecol,0,0,2);
                                        drawtilemasknew(spritecode,spritex+8,spritey,spritecol,0,0,2);
                                        drawtilemasknew(spritecode+3,spritex,spritey+8,spritecol,0,0,2);
                                        drawtilemasknew(spritecode+1,spritex+8,spritey+8,spritecol,0,0,2);
                                }
                                else
                                {
                                        drawtilemasknew(spritecode+2,spritex,spritey+8,spritecol,0,1,2);
                                        drawtilemasknew(spritecode,spritex+8,spritey+8,spritecol,0,1,2);
                                        drawtilemasknew(spritecode+3,spritex,spritey,spritecol,0,1,2);
                                        drawtilemasknew(spritecode+1,spritex+8,spritey,spritecol,0,1,2);
                                }
                        }
                        else
                        {
                                if (!yflip)
                                {
                                        drawtilemasknew(spritecode+2,spritex+8,spritey,spritecol,1,0,2);
                                        drawtilemasknew(spritecode,spritex,spritey,spritecol,1,0,2);
                                        drawtilemasknew(spritecode+3,spritex+8,spritey+8,spritecol,1,0,2);
                                        drawtilemasknew(spritecode+1,spritex,spritey+8,spritecol,1,0,2);
                                }
                                else
                                {
                                        drawtilemasknew(spritecode+2,spritex+8,spritey+8,spritecol,1,1,2);
                                        drawtilemasknew(spritecode,spritex,spritey+8,spritecol,1,1,2);
                                        drawtilemasknew(spritecode+3,spritex+8,spritey,spritecol,1,1,2);
                                        drawtilemasknew(spritecode+1,spritex,spritey,spritecol,1,1,2);
                                }
                        }
                }
        }
}

void anteaterdraw()
{
        int x,y;
        int pos=0x8800;
        int scroll;
        for (x=0;x<32;x++)
        {
                for (y=0;y<32;y++)
                {
                        scroll=(signed char)ram[0x9000+(y<<1)];
                        drawtilenew(ram[pos++],(((31-x)<<3)+scroll)&255,y<<3,ram[0x9001+(y<<1)]&7,0,0,2);
                }
        }
        anteaterdrawsprites();
        blit(buffer,screen,16,0,0,0,224,256);
}

void anteaterrun()
{
        while (!keys[KEY_ESC])
        {
                if (mz80exec(51200)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                anteaterdraw();
                if (nmi)
                   mz80nmi();
        }
}