/*YAAME - Bombjack driver

  CPU : Z80 (4 mhz)
  CPU : Z80 (2 mhz) (not emulated)
  Sound : 3xAY whatever (not emulated)
  Graphics : 256x256, 128 colours

  Known problems :

  Occasional graphics bug
*/

#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "yaame.h"

unsigned char bjbackground[4096];

int bombjackpalchange=1;
int bjbackgrnd;

void bombjackpal(UINT32 addr, UINT8 val, struct MemoryWriteByte *pMemWrite)
{
        int colour,r,g,b,bit0,bit1,bit2,bit3;
        ram[addr]=val;

	val = ram[addr & ~1];
        r=(val&0xF)<<2;
        g=(val>>4)<<2;
	val = ram[addr | 1];
        b=(val&0xF)<<2;
        colour=(addr&255)>>1;
        yaamepal[colour].r=r;
        yaamepal[colour].g=g;
        yaamepal[colour].b=b;
        bombjackpalchange=1;
}

void bombjackbackgrnd(UINT32 addr, UINT8 val, struct MemoryWriteByte *pMemWrite)
{
        bjbackgrnd=val;
}

UINT8 bombjackreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0;
        if (keys[KEY_UP])
           temp|=4;
        if (keys[KEY_DOWN])
           temp|=8;
        if (keys[KEY_LEFT])
           temp|=2;
        if (keys[KEY_RIGHT])
           temp|=1;
        if (keys[KEY_SPACE])
           temp|=16;
        return temp;
}

UINT8 bombjackreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        return 0;
}

UINT8 bombjackreadin2(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xF0;
        if (keys[KEY_C])
           temp|=1;
        if (keys[KEY_1])
           temp|=4;
        return temp;
}

UINT8 bombjackreaddsw1(UINT32 addr, struct MemoryReadByte *m)
{
        return 0;
}

UINT8 bombjackreaddsw2(UINT32 addr, struct MemoryReadByte *m)
{
        return 0;
}

struct MemoryReadByte bombjackread[] =
{
        {0xB000,0xB000,bombjackreadin0},
        {0xB001,0xB001,bombjackreadin1},
        {0xB002,0xB002,bombjackreadin2},
        {0xB004,0xB004,bombjackreaddsw1},
        {0xB005,0xB005,bombjackreaddsw2},
        {-1,-1,NULL}
};

struct MemoryWriteByte bombjackwrite[] =
{
        {0,0x4000,writerom},
        {0x9E00,0x9E00,bombjackbackgrnd},
        {0x9C00,0x9D00,bombjackpal},
        {0xB000,0xB000,nmienable},
        {-1,-1,NULL}
};

struct z80PortRead bombjackportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite bombjackportwrite[] =
{
	{-1,	-1,	NULL}
};

int bombjackloadgfx()
{
        FILE *f,*ff,*fff;
        int y,c;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read3plane(0,512,"03_e08t.bin","04_h08t.bin","05_k08t.bin");
        read3plane2(512,1024,"14_j07b.bin","15_l07b.bin","16_m07b.bin");
        read3plane(1536,1024,"06_l08t.bin","07_n08t.bin","08_r08t.bin");
        f=fopen("02_p04t.bin","rb");
        fread(bjbackground,4096,1,f);
        fclose(f);
        return 0;
}

int bombjackinit()
{
        printf("Loading Bombjack ROMs\n");
        loadrom("09_j01b.bin",0,8192);
        loadrom("10_l01b.bin",0x2000,8192);
        loadrom("11_m01b.bin",0x4000,8192);
        loadrom("12_n01b.bin",0x6000,8192);
        loadrom("13.1r",0xC000,8192);
        bombjackloadgfx();
        initz80(bombjackread,bombjackwrite,bombjackportread,bombjackportwrite);
        printf("Bombjack loaded\n");
        nmi=0;
        return 0;
}

void bombjackdraw()
{
        int x,y;
        int c;
        int tile;
        int colour;
        int pos=0x9000,pos2=0x9400,pos3=0x200*(bjbackgrnd&7),pos4;
        pos4=pos3+0x100;
        if (bjbackgrnd&0x10)
        {
                for (x=15;x>-1;x--)
                {
                        for (y=0;y<16;y++)
                        {
                                drawtilenew((bjbackground[pos3]<<2)+1536,(x<<4)+8,(y<<4),bjbackground[pos4],0,0,3);
                                drawtilenew((bjbackground[pos3]<<2)+1537,(x<<4)+8,(y<<4)+8,bjbackground[pos4],0,0,3);
                                drawtilenew((bjbackground[pos3]<<2)+1538,(x<<4),(y<<4),bjbackground[pos4],0,0,3);
                                drawtilenew((bjbackground[pos3]<<2)+1539,(x<<4),(y<<4)+8,bjbackground[pos4],0,0,3);
                                pos3++;
                                pos4++;
                        }
                }
        }
        for (x=0;x<32;x++)
        {
                for (y=0;y<32;y++)
                {
                        if (bjbackgrnd&0x10)
                        {
                                if (ram[pos2]&0x10)
                                   drawtilemasknew(ram[pos++]+256,(31-x)*8,y*8,ram[pos2++]&15,0,0,3);
                                else
                                   drawtilemasknew(ram[pos++],(31-x)*8,y*8,ram[pos2++]&15,0,0,3);
                        }
                        else
                        {
                                if (ram[pos2]&0x10)
                                   drawtilenew(ram[pos++]+256,(31-x)<<3,y<<3,ram[pos2++]&15,0,0,3);
                                else
                                   drawtilenew(ram[pos++],(31-x)<<3,y<<3,ram[pos2++]&15,0,0,3);
                        }
                }
        }
        pos=0x9820;
        for (c=0;c<24;c++)
        {
                x=ram[pos+2]-1;
                y=ram[pos+3];
                if (x>8&&y>8)
                {
                        tile=ram[pos]&0x7F;
                        colour=ram[pos+1]&0xF;
                        if (!(ram[pos]&0x80))
                        {
                                drawtilemasknew((tile<<2)+512,x+8,y,colour,0,0,3);
                                drawtilemasknew((tile<<2)+513,x+8,y+8,colour,0,0,3);
                                drawtilemasknew((tile<<2)+514,x,y,colour,0,0,3);
                                drawtilemasknew((tile<<2)+515,x,y+8,colour,0,0,3);
                        }
                        else
                        {
                                tile&=31;
                                drawtilemasknew((tile<<4)+1024,x+8+16,y,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1025,x+8+16,y+8,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1026,x+16,y,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1027,x+16,y+8,colour,0,0,3);

                                drawtilemasknew((tile<<4)+1028,x+8+16,y+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1029,x+8+16,y+8+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1030,x+16,y+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1031,x+16,y+8+16,colour,0,0,3);

                                drawtilemasknew((tile<<4)+1032,x+8,y,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1033,x+8,y+8,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1034,x,y,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1035,x,y+8,colour,0,0,3);

                                drawtilemasknew((tile<<4)+1036,x+8,y+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1037,x+8,y+8+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1038,x,y+16,colour,0,0,3);
                                drawtilemasknew((tile<<4)+1039,x,y+8+16,colour,0,0,3);
                        }
                }
                pos+=4;
        }
        if (bombjackpalchange)
        {
                setpal(yaamepal);
                bombjackpalchange=0;
        }
//        waitforsync();
        blit(buffer,screen,0,0,0,0,256,256);
}

void bombjackrun()
{
        while (!keys[KEY_ESC])
        {
                if (mz80exec(51200)!=0x80000000)
                {
                        mz80GetContext(&z80);
                        closegfx();
                        printf("It crashed at %04X\n",z80.z80pc);
                        exit(-1);
                }
                bombjackdraw();
                if (nmi)
                   mz80nmi();
        }
}