/*YAAME - Common driver code (graphics loading etc)*/

#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "m6502.h"
#include "yaame.h"

unsigned char tiles[16384][8][8];
unsigned char ram2[65536];
struct mz80context z802;
volatile int yaamedraw;
int temp;
volatile void drawupdate()
{
        yaamedraw=1;
}

END_OF_FUNCTION(drawupdate);

void writerom(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
}

void nmienable(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        nmi=data;
}

void loadrom(char *fn, int addr, int size)
{
        FILE *f=fopen(fn,"rb");
        if (!f)
        {
                printf("File %s not found\n",fn);
                chdir("..");
                exit(-1);
        }
        fread(ram+addr,size,1,f);
        fclose(f);
}

/*Draws a tile onto the back buffer from the bitplane structure*/
void drawtile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((xx+y)>0&&(x+yy)>0)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void draw3bpptile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2,byte3;
        for (yy=0;yy<8;yy++)
        {
                byte3=bitplane[0][tile][yy];
                byte2=bitplane[1][tile][yy];
                byte1=bitplane[2][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((xx+y)>0&&(x+yy)>0)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|((byte3&1)<<2)|(colset<<3);
                        byte1>>=1;
                        byte2>>=1;
                        byte3>>=1;
                }
        }
}

void draw3bpptilemask(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2,byte3;
        for (yy=0;yy<8;yy++)
        {
                byte3=bitplane[0][tile][yy];
                byte2=bitplane[1][tile][yy];
                byte1=bitplane[2][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((byte1&1)|((byte2&1)<<1)|((byte3&1)<<2))
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|((byte3&1)<<2)|(colset<<3);
                        byte1>>=1;
                        byte2>>=1;
                        byte3>>=1;
                }
        }
}

void draw4bpptile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2,byte3,byte4;
        for (yy=0;yy<8;yy++)
        {
                byte4=bitplane[0][tile][yy];
                byte3=bitplane[1][tile][yy];
                byte2=bitplane[2][tile][yy];
                byte1=bitplane[3][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((xx+y)>0&&(x+yy)>0)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|((byte3&1)<<2)|((byte4&1)<<3)|(colset<<4);
                        byte1>>=1;
                        byte2>>=1;
                        byte3>>=1;
                        byte4>>=1;
                }
        }
}

/*Draws a tile onto the back buffer from the bitplane structure with masking*/
void drawtilemask(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void drawsprite(int tile, unsigned char x, unsigned int y, unsigned char colset, int xflip, int yflip)
{
        int xx,yy;
        unsigned char byte1,byte2;
        if (!xflip)
        {
                for (yy=0;yy<8;yy++)
                {
                        byte2=bitplane[0][tile][yy];
                        byte1=bitplane[1][tile][yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
        else
        {
                for (yy=7;yy>-1;yy--)
                {
                        byte2=bitplane[0][tile][7-yy];
                        byte1=bitplane[1][tile][7-yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
}

void draw3bppsprite(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2,byte3;
        for (yy=0;yy<8;yy++)
        {
                byte3=bitplane[0][tile][yy];
                byte2=bitplane[1][tile][yy];
                byte1=bitplane[2][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((byte1&1)|((byte2&1)<<1)|((byte3&1)<<2))
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|((byte3&1)<<2)|(colset<<3);
                        byte1>>=1;
                        byte2>>=1;
                        byte3>>=1;
                }
        }
}

void drawtile2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void drawtilemask2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void drawtilenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

void drawtilemasknew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

void drawspritenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
      int xx,yy;
      if (!yflip)
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
      else
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
}

void initz80(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite)
{
        memset(&z80,0,sizeof(z80));
        z80.z80Base=ram;
        z80.z80IoRead=portread;
        z80.z80IoWrite=portwrite;
        z80.z80MemRead=read;
        z80.z80MemWrite=write;
	mz80SetContext(&z80);
	mz80reset();
	mz80GetContext(&z80);
	z80.z80intAddr=0x38;
	z80.z80nmiAddr=0x66;
	mz80SetContext(&z80);
}

void init8080(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite)
{
        memset(&z80,0,sizeof(z80));
        z80.z80Base=ram;
        z80.z80IoRead=portread;
        z80.z80IoWrite=portwrite;
        z80.z80MemRead=read;
        z80.z80MemWrite=write;
	mz80SetContext(&z80);
	mz80reset();
	mz80GetContext(&z80);
	z80.z80intAddr=0x8;
	z80.z80nmiAddr=0x10;
	mz80SetContext(&z80);
}

void initz802(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite)
{
        memset(&z802,0,sizeof(z802));
        z802.z80Base=ram2;
        z802.z80IoRead=portread;
        z802.z80IoWrite=portwrite;
        z802.z80MemRead=read;
        z802.z80MemWrite=write;
	mz80SetContext(&z802);
	mz80reset();
	mz80GetContext(&z802);
	z802.z80intAddr=0x38;
	z802.z80nmiAddr=0x66;
	mz80SetContext(&z802);
}

void loadrom2(char *fn, int addr, int size)
{
        FILE *f=fopen(fn,"rb");
        if (!f)
        {
                printf("File %s not found\n",fn);
                chdir("..");
                exit(-1);
        }
        fread(ram2+addr,size,1,f);
        fclose(f);
}

void init6502(struct MemoryReadByte *read, struct MemoryWriteByte *write)
{
        memset(&m6502,0,sizeof(m6502));
        m6502init();
        m6502.m6502Base=ram;
        m6502.m6502MemoryRead=read;
        m6502.m6502MemoryWrite=write;
        m6502SetContext(&m6502);
        m6502reset();

//        m6502SetContext(&m6502);
//	     m6502reset();
}

void inityaame()
{
        yaamedraw=0;
        LOCK_VARIABLE(yaamedraw);
        LOCK_FUNCTION(drawupdate);
        install_int_ex(drawupdate,BPS_TO_TIMER(60));
}

void waitforsync()
{
        while (!yaamedraw);
        yaamedraw=0;
}

void read2plane(int start, int num, char *fn1, char *fn2)
{
      FILE *f1,*f2;
      int c,y,x,dat1,dat2,col;
      f1=fopen(fn1,"rb");
      f2=fopen(fn2,"rb");
      for (c=0;c<num;c++)
      {
            for (y=0;y<8;y++)
            {
                  dat1=getc(f1);
                  dat2=getc(f2);
                  for (x=0;x<8;x++)
                  {
                        col=0;
                        if (dat1&1)
                           col|=2;
                        if (dat2&1)
                           col|=1;
                        tiles[c+start][y][(7-x)]=col;
                        dat1>>=1;
                        dat2>>=1;
                  }
            }
      }
      fclose(f2);
      fclose(f1);
}

void read2planeflipped(int start, int num, char *fn1, char *fn2)
{
      FILE *f1,*f2;
      int c,y,x,dat1,dat2,col;
      f1=fopen(fn1,"rb");
      f2=fopen(fn2,"rb");
      for (c=0;c<num;c++)
      {
            for (y=0;y<8;y++)
            {
                  dat1=getc(f1);
                  dat2=getc(f2);
                  for (x=0;x<8;x++)
                  {
                        col=0;
                        if (dat1&1)
                           col|=2;
                        if (dat2&1)
                           col|=1;
                        tiles[c+start][7-y][7-x]=col;
                        dat1>>=1;
                        dat2>>=1;
                  }
            }
      }
      fclose(f2);
      fclose(f1);
}

void read3plane(int start, int num, char *fn1, char *fn2, char *fn3)
{
      FILE *f1,*f2,*f3;
      int c,y,x,dat1,dat2,dat3,col;
      f1=fopen(fn1,"rb");
      f2=fopen(fn2,"rb");
      f3=fopen(fn3,"rb");
      for (c=0;c<num;c++)
      {
            for (y=0;y<8;y++)
            {
                  dat1=getc(f1);
                  dat2=getc(f2);
                  dat3=getc(f3);
                  for (x=0;x<8;x++)
                  {
                        col=0;
                        if (dat1&1)
                           col|=4;
                        if (dat2&1)
                           col|=2;
                        if (dat3&1)
                           col|=1;
                        tiles[c+start][7-y][7-x]=col;
                        dat1>>=1;
                        dat2>>=1;
                        dat3>>=1;
                  }
            }
      }
      fclose(f3);
      fclose(f2);
      fclose(f1);
}

void read3plane2(int start, int num, char *fn1, char *fn2, char *fn3)
{
      FILE *f1,*f2,*f3;
      int c,y,x,dat1,dat2,dat3,col;
      f1=fopen(fn1,"rb");
      f2=fopen(fn2,"rb");
      f3=fopen(fn3,"rb");
      for (c=0;c<num;c++)
      {
            for (y=0;y<8;y++)
            {
                  dat1=getc(f1);
                  dat2=getc(f2);
                  dat3=getc(f3);
                  for (x=0;x<8;x++)
                  {
                        col=0;
                        if (dat1&1)
                           col|=1;
                        if (dat2&1)
                           col|=2;
                        if (dat3&1)
                           col|=4;
                        tiles[c+start][7-y][7-x]=col;
                        dat1>>=1;
                        dat2>>=1;
                        dat3>>=1;
                  }
            }
      }
      fclose(f3);
      fclose(f2);
      fclose(f1);
}
