/*YAAME - Galaxian driver
  Also supports War Of The Bugs and Pac-man (bootleg)

  CPU : Z80 (3.57 mhz)
  Graphics : 232x256 in 32 colours

  Known problems :

  Background wave sound channel not emulated
*/

#include <stdio.h>
#include "gfx.h"
#include "mem.h"
#include "mz80.h"
#include "yaame.h"

int pacman;

static unsigned char samples[5000] =	/* a simple sine (sort of) wave */
{
	0x00,0x00,0x00,0x00,0x22,0x22,0x22,0x22,0x44,0x44,0x44,0x44,0x22,0x22,0x22,0x22,
	0x00,0x00,0x00,0x00,0xdd,0xdd,0xdd,0xdd,0xbb,0xbb,0xbb,0xbb,0xdd,0xdd,0xdd,0xdd
};

SAMPLE *sine;
SAMPLE *noise;

static unsigned char galaxiancolour[] =
{
	0x00,0x00,0x00,0xF6,
	0x00,0x16,0xC0,0x3F,
	0x00,0xD8,0x07,0x3F,
	0x00,0xC0,0xC4,0x07,
	0x00,0xC0,0xA0,0x0C,
	0x00,0x00,0x00,0x07,
	0x00,0xF6,0x07,0xF0,
	0x00,0x76,0x07,0xC6
};

static unsigned char warofbugcolour[] =
{
	0x00,0xFF,0x07,0xC0,
	0x00,0x07,0xC0,0x3F,
	0x00,0x38,0x07,0xC0,
	0x00,0x07,0xC0,0x38,
	0x00,0x3F,0x38,0x07,
	0x00,0xC0,0x3F,0x07,
	0x00,0xF8,0x07,0x38,
	0x00,0xC0,0x38,0xC7,
};

static void galaxiansoundwrite(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        if (data&&data!=0xFF)
           adjust_sample(sine,255,127,(48000/(256-data))*1000,TRUE);
        else
           adjust_sample(sine,0,127,1000,TRUE);
}

static void galaxiannoise(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        if (data&1)
           adjust_sample(noise,255,127,1000,TRUE);
        else
           adjust_sample(noise,0,127,1000,TRUE);
}

struct MemoryWriteByte galaxianwrite[] =
{
        {0,0x2800,writerom},
        {0x6803,0x6803,galaxiannoise},
        {0x7001,0x7001,nmienable},
        {0x7800,0x7800,galaxiansoundwrite},
        {-1,-1,NULL}
};

UINT8 galaxianreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char result=0;
        if (keys[KEY_C])
           result|=1;
        if (keys[KEY_LEFT])
           result|=4;
        if (keys[KEY_RIGHT])
           result|=8;
        if (keys[KEY_SPACE])
           result|=16;
        if (pacman)
        {
                if (keys[KEY_UP])
                   result|=128;
                if (keys[KEY_DOWN])
                   result|=32;
        }
        return result;
}

UINT8 galaxianreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char result=0;
        if (keys[KEY_1])
           result|=1;
        return result;
}

struct MemoryReadByte galaxianread[] =
{
        {0x6000,0x6000,galaxianreadin0},
        {0x6800,0x6800,galaxianreadin1},
        {-1,-1,NULL}
};

struct z80PortRead galaxianportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite galaxianportwrite[] =
{
	{-1,	-1,	NULL}
};

int galaxianloadgfx()
{
        FILE *f,*ff;
        int x,y,c;
        unsigned char byte1,byte2;
        read2planeflipped(0,256,"galmidw.1j","galmidw.1k");
        for (c=0;c<32;c++)
	     {
		         int bit0,bit1,bit2;
		         bit0 = (galaxiancolour[c] >> 0) & 0x01;
		         bit1 = (galaxiancolour[c] >> 1) & 0x01;
		         bit2 = (galaxiancolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (galaxiancolour[c] >> 3) & 0x01;
		         bit1 = (galaxiancolour[c] >> 4) & 0x01;
		         bit2 = (galaxiancolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (galaxiancolour[c] >> 6) & 0x01;
		         bit2 = (galaxiancolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int warofbugloadgfx()
{
        FILE *f,*ff;
        int x,y,c;
        unsigned char byte1,byte2;
        read2planeflipped(0,256,"warofbug.1j","warofbug.1k");
	     for (c=0;c<32;c++)
	     {
		         int bit0,bit1,bit2;
		         bit0 = (warofbugcolour[c] >> 0) & 0x01;
		         bit1 = (warofbugcolour[c] >> 1) & 0x01;
		         bit2 = (warofbugcolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (warofbugcolour[c] >> 3) & 0x01;
		         bit1 = (warofbugcolour[c] >> 4) & 0x01;
		         bit2 = (warofbugcolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (warofbugcolour[c] >> 6) & 0x01;
		         bit2 = (warofbugcolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int pacblloadgfx()
{
        FILE *f,*ff;
        int x,y,c;
        unsigned char byte1,byte2;
        read2planeflipped(0,256,"blpac12b","blpac11b");
        read2planeflipped(256,256,"blpac10b","blpac9b");
        memset(yaamepal,0,768);
	     for (c=0;c<32;c++)
	     {
		         int bit0,bit1,bit2;
		         bit0 = (galaxiancolour[c] >> 0) & 0x01;
		         bit1 = (galaxiancolour[c] >> 1) & 0x01;
		         bit2 = (galaxiancolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (galaxiancolour[c] >> 3) & 0x01;
		         bit1 = (galaxiancolour[c] >> 4) & 0x01;
		         bit2 = (galaxiancolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (galaxiancolour[c] >> 6) & 0x01;
		         bit2 = (galaxiancolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int galaxianinit()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading Galaxian ROMs\n");
        loadrom("galmidw.u",0,2048);
        loadrom("galmidw.v",0x800,2048);
        loadrom("galmidw.w",0x1000,2048);
        loadrom("galmidw.y",0x1800,2048);
        loadrom("galmidw.z",0x2000,2048);
        galaxianloadgfx();
        initz80(galaxianread,galaxianwrite,galaxianportread,galaxianportwrite);
        printf("Galaxian loaded\n");
        pacman=0;
        return 0;
}

int warofbuginit()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading War Of The Bugs ROMs\n");
        loadrom("warofbug.u",0,2048);
        loadrom("warofbug.v",0x800,2048);
        loadrom("warofbug.w",0x1000,2048);
        loadrom("warofbug.y",0x1800,2048);
        loadrom("warofbug.z",0x2000,2048);
        warofbugloadgfx();
        initz80(galaxianread,galaxianwrite,galaxianportread,galaxianportwrite);
        printf("War Of The Bugs loaded\n");
        pacman=0;
        return 0;
}

int pacmanblinit()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading Pac-man bootleg ROMs\n");
        loadrom("blpac1b",0,2048);
        loadrom("blpac2b",0x800,2048);
        loadrom("blpac3b",0x1000,2048);
        loadrom("blpac4b",0x1800,2048);
        loadrom("blpac5b",0x2000,2048);
        loadrom("blpac6b",0x2800,2048);
        loadrom("blpac7b",0x3000,2048);
        pacblloadgfx();
        initz80(galaxianread,galaxianwrite,galaxianportread,galaxianportwrite);
        printf("Pac-man bootleg loaded\n");
        pacman=1;
        return 0;
}

void galaxiandrawback()
{
        int y,x;
        int pos=0x5000;
        unsigned char c;
        signed char scroll;
        for (x = 32; x>0; x--)
        {
                for (y = 0; y < 32; y++)
                {
                        c = ram[pos++];
                        scroll=(signed char)ram[0x5800+(y<<1)];
//                        if (((x<<3)+scroll)>0&&((x<<3)+scroll)<248)
                           drawtilenew(c,((x<<3)+scroll)&255,y<<3,ram[0x5801+(y<<1)],0,0,2);
                }
        }
}

void galaxiandrawsprites()
{
        int cc,ccc;
        int spritex,spritey,spritecode,xflip,yflip,spritecol;
        for (cc=7;cc>-1;cc--)
        {
                ccc=cc<<2;
                spritex=ram[0x5840+ccc]+8;
                if (spritex>8 && spritex<0xF8)
                {
                        spritey=ram[0x5843+ccc]+1;
                        spritecol=ram[0x5842+ccc]&7;
                        spritecode=ram[0x5841+ccc]&0x3F;
                        spritecode<<=2;
                        if (pacman)
                        {
                                spritecode+=256;
                        }
                        xflip=ram[0x5841+ccc]&0x80;
                        yflip=ram[0x5841+ccc]&0x40;
                        if (!xflip)
                        {
                                if (!yflip)
                                {
                                        drawtilemasknew(spritecode+2,spritex,spritey,spritecol,0,0,2);
                                        drawtilemasknew(spritecode,spritex+8,spritey,spritecol,0,0,2);
                                        drawtilemasknew(spritecode+3,spritex,spritey+8,spritecol,0,0,2);
                                        drawtilemasknew(spritecode+1,spritex+8,spritey+8,spritecol,0,0,2);
                                }
                                else
                                {
                                        drawtilemasknew(spritecode+2,spritex,spritey+8,spritecol,0,1,2);
                                        drawtilemasknew(spritecode,spritex+8,spritey+8,spritecol,0,1,2);
                                        drawtilemasknew(spritecode+3,spritex,spritey,spritecol,0,1,2);
                                        drawtilemasknew(spritecode+1,spritex+8,spritey,spritecol,0,1,2);
                                }
                        }
                        else
                        {
                                if (!yflip)
                                {
                                        drawtilemasknew(spritecode+2,spritex+8,spritey,spritecol,1,0,2);
                                        drawtilemasknew(spritecode,spritex,spritey,spritecol,1,0,2);
                                        drawtilemasknew(spritecode+3,spritex+8,spritey+8,spritecol,1,0,2);
                                        drawtilemasknew(spritecode+1,spritex,spritey+8,spritecol,1,0,2);
                                }
                                else
                                {
                                        drawtilemasknew(spritecode+2,spritex+8,spritey+8,spritecol,1,1,2);
                                        drawtilemasknew(spritecode,spritex,spritey+8,spritecol,1,1,2);
                                        drawtilemasknew(spritecode+3,spritex+8,spritey,spritecol,1,1,2);
                                        drawtilemasknew(spritecode+1,spritex,spritey,spritecol,1,1,2);
                                }
                        }
                }
        }
}

void galaxiandrawbullets()
{
        int x,y,cc;
        for (cc=0;cc<32;cc+=4)
        {
                x=ram[0x5861+cc];
                y=256-ram[0x5863+cc]-4;
                if (x>0&&x<256&&y>-1&&y<253)
                {
                        if (cc!=28)
                        {
                                buffer->line[y][x]=3;
                                buffer->line[y+1][x]=3;
                                buffer->line[y+2][x]=3;
                        }
                        else
                        {
                                buffer->line[y][x+8]=7;
                                buffer->line[y+1][x+8]=7;
                                buffer->line[y+2][x+8]=7;
                        }
                }
        }
}

void galaxiandraw()
{
        int c;
        char s[40];
        galaxiandrawback();
        galaxiandrawsprites();
        galaxiandrawbullets();
        waitforsync();
        blit(buffer,screen,16,0,0,0,232,256);
}

void galaxianrun()
{
        int c;
        sine=create_sample(8,0,64,32);
        if (sine)
        {
               memcpy(sine->data,samples,32);
        }
        noise=create_sample(8,0,5000,5000);
        if (noise)
        {
               for (c=0;c<5000;c++)
                   samples[c]=random()&255;
               memcpy(noise->data,samples,5000);
        }
        install_sound(DIGI_AUTODETECT,MIDI_NONE,0);
        play_sample(sine,255,127,1000*200,TRUE);
        play_sample(noise,255,127,1000,TRUE);
        while (!keys[KEY_ESC])
        {
                if (mz80exec(51200)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                galaxiandraw();
                if (nmi)
                   mz80nmi();
        }
}
