/*YAAME - Donkey Kong driver
  Also supports Radar Scope

  CPU : Z80 (3.57 mhz)
  CPU : i8035 (not emulated)
  Graphics : 256x256,64 colours

  Known problems :

  Radar Scope colours are probably wrong, and the star background/grid and
  some sprites are missing, but I don't really care because it's a crap game
  anyway
*/

#include <stdio.h>
#include "gfx.h"
#include "mem.h"
#include "mz80.h"
#include "yaame.h"

unsigned char intenable;
unsigned char palbank;

static void writeint(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        intenable=data;
}

static void kongwritepal(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        if (addr&1)
        {
                palbank&=1;
                if (data&1)
                   palbank|=2;
        }
        else
        {
                palbank&=2;
                if (data&1)
                   palbank|=1;
        }
}

struct MemoryWriteByte kongwrite[] =
{
        {0,0x4000,writerom},
        {0x7D84,0x7D84,writeint},
        {0x7D86,0x7D87,kongwritepal},
        {-1,-1,NULL}
};

UINT8 readkongin0(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0;
        if (keys[KEY_DOWN])
           temp|=8;
        if (keys[KEY_UP])
           temp|=4;
        if (keys[KEY_LEFT])
           temp|=2;
        if (keys[KEY_RIGHT])
           temp|=1;
        if (keys[KEY_SPACE])
           temp|=16;
        return temp;
}

UINT8 readkongin1(UINT32 addr, struct MemoryReadByte *crap)
{
        return 0;
}

UINT8 readkongin2(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0;
        if (keys[KEY_C])
           temp|=128;
        if (keys[KEY_1])
           temp|=4;
        return temp;
}

UINT8 readkongdsw1(UINT32 addr, struct MemoryReadByte *crap)
{
        return 0x84;
}

struct MemoryReadByte kongread[] =
{
        {0x7C00,0x7C7F,readkongin0},
        {0x7C80,0x7CFF,readkongin1},
        {0x7D00,0x7D7F,readkongin2},
        {0x7D80,0x7DFF,readkongdsw1},
        {-1,-1,NULL}
};

struct z80PortRead kongportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite kongportwrite[] =
{
	{-1,	-1,	NULL}
};

static unsigned char dkongpal[] =
{
	/* 2k - palette low 4 bits (inverted) */
	0xFF,0xFC,0xF0,0xFF,0xFF,0xFB,0xFF,0xF0,0xFF,0xFA,0xFF,0xFD,0xFF,0xFC,0xFF,0xF3,
	0xFF,0xF3,0xFB,0xFF,0xFF,0xF0,0xFD,0xF3,0xFF,0xFF,0xFC,0xF0,0xFF,0xFF,0xFA,0xF0,
	0xFF,0xFF,0xFA,0xF3,0xFF,0xF0,0xF3,0xF5,0xFF,0xFD,0xF0,0xF5,0xFF,0xFC,0xF3,0xFA,
	0xFF,0xF0,0xF0,0xFC,0xFF,0xFF,0xFB,0xF3,0xFF,0xFE,0xFA,0xFC,0xFF,0xFB,0xFF,0xFF,
	0xFF,0xFC,0xF0,0xFF,0xFF,0xFB,0xFF,0xF0,0xFF,0xFA,0xFF,0xFD,0xFF,0xF3,0xFA,0xF0,
	0xFF,0xF3,0xFA,0xF0,0xFF,0xF3,0xFA,0xF0,0xFF,0xF3,0xFA,0xF0,0xFF,0xFF,0xFA,0xF0,
	0xFF,0xFF,0xFA,0xF3,0xFF,0xF0,0xF3,0xF5,0xFF,0xFD,0xF0,0xF5,0xFF,0xFC,0xF3,0xFA,
	0xFF,0xF0,0xF0,0xFC,0xFF,0xFF,0xFB,0xF3,0xFF,0xFE,0xFA,0xFC,0xFF,0xFB,0xFF,0xFF,
	0xFF,0xFC,0xF0,0xFF,0xFF,0xFB,0xFF,0xF0,0xFF,0xFA,0xFF,0xFD,0xFF,0xFA,0xFF,0xF0,
	0xFF,0xFA,0xFF,0xF0,0xFF,0xFA,0xFF,0xF0,0xFF,0xFA,0xFF,0xF0,0xFF,0xFF,0xFA,0xF0,
	0xFF,0xFF,0xFA,0xF3,0xFF,0xF0,0xF3,0xF5,0xFF,0xFD,0xF0,0xF5,0xFF,0xFC,0xF3,0xFA,
	0xFF,0xF0,0xF0,0xFC,0xFF,0xFF,0xFB,0xF3,0xFF,0xFE,0xFA,0xFC,0xFF,0xFB,0xFF,0xFF,
	0xFF,0xFC,0xF0,0xFF,0xFF,0xFB,0xFF,0xF0,0xFF,0xFA,0xFF,0xFD,0xFF,0xFC,0xF0,0xFB,
	0xFF,0xFC,0xF0,0xFB,0xFF,0xFC,0xF0,0xFB,0xFF,0xFC,0xF0,0xFB,0xFF,0xFF,0xFA,0xF0,
	0xFF,0xFF,0xFA,0xF3,0xFF,0xF0,0xF3,0xF5,0xFF,0xFD,0xF0,0xF5,0xFF,0xFC,0xF3,0xFA,
	0xFF,0xF0,0xF0,0xFC,0xFF,0xFF,0xFB,0xF3,0xFF,0xFE,0xFA,0xFC,0xFF,0xFB,0xFF,0xFF,
	/* 2j - palette high 4 bits (inverted) */
	0xFF,0xF6,0xFE,0xF1,0xFF,0xF0,0xF1,0xF0,0xFF,0xF0,0xF1,0xFF,0xFF,0xFF,0xF1,0xFE,
	0xFF,0xF1,0xF0,0xF1,0xFF,0xF0,0xF1,0xF0,0xFF,0xF1,0xFF,0xFE,0xFF,0xF5,0xF0,0xF0,
	0xFF,0xF5,0xF0,0xF1,0xFF,0xF0,0xF1,0xF1,0xFF,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF0,
	0xFF,0xF0,0xFE,0xFF,0xFF,0xF1,0xF0,0xF1,0xFF,0xF2,0xF0,0xFF,0xFF,0xF0,0xF1,0xFF,
	0xFF,0xF6,0xFE,0xF1,0xFF,0xF0,0xF1,0xF0,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF0,0xF0,
	0xFF,0xF1,0xF0,0xF0,0xFF,0xF1,0xF0,0xF0,0xFF,0xF1,0xF0,0xF0,0xFF,0xF5,0xF0,0xF0,
	0xFF,0xF5,0xF0,0xF1,0xFF,0xF0,0xF1,0xF1,0xFF,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF0,
	0xFF,0xF0,0xFE,0xFF,0xFF,0xF1,0xF0,0xF1,0xFF,0xF2,0xF0,0xFF,0xFF,0xF0,0xF1,0xFF,
	0xFF,0xF6,0xFE,0xF1,0xFF,0xF0,0xF1,0xF0,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF7,0xFE,
	0xFF,0xF1,0xF7,0xFE,0xFF,0xF1,0xF7,0xFE,0xFF,0xF1,0xF7,0xFE,0xFF,0xF5,0xF0,0xF0,
	0xFF,0xF5,0xF0,0xF1,0xFF,0xF0,0xF1,0xF1,0xFF,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF0,
	0xFF,0xF0,0xFE,0xFF,0xFF,0xF1,0xF0,0xF1,0xFF,0xF2,0xF0,0xFF,0xFF,0xF0,0xF1,0xFF,
	0xFF,0xF6,0xFE,0xF1,0xFF,0xF0,0xF1,0xF0,0xFF,0xF0,0xF1,0xFF,0xFF,0xFF,0xFE,0xF0,
	0xFF,0xFF,0xFE,0xF0,0xFF,0xFF,0xFE,0xF0,0xFF,0xFF,0xFE,0xF0,0xFF,0xF5,0xF0,0xF0,
	0xFF,0xF5,0xF0,0xF1,0xFF,0xF0,0xF1,0xF1,0xFF,0xFF,0xF0,0xF1,0xFF,0xFF,0xF1,0xF0,
	0xFF,0xF0,0xFE,0xFF,0xFF,0xF1,0xF0,0xF1,0xFF,0xF2,0xF0,0xFF,0xFF,0xF0,0xF1,0xFF,
	/* 5f - character color codes on a per-column basis */
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3,
	0xF0,0xF1,0xF1,0xF2,0xF6,0xF6,0xF4,0xF6,0xF6,0xF6,0xF6,0xF3,0xF6,0xF3,0xF4,0xF3,
	0xF6,0xF6,0xF6,0xF6,0xF4,0xF3,0xF4,0xF5,0xF4,0xF6,0xF5,0xF3,0xF5,0xF3,0xF6,0xF3
};

int kongloadgfx()
{
        int c;
        buffer=create_bitmap(512,512);
        clear(buffer);
        read2planeflipped(0,256,"dk.3p","dk.3n");
        read2planeflipped(256,256,"dk.7e","dk.7c");
        read2planeflipped(512,256,"dk.7f","dk.7d");
	     for (c = 0;c < 256;c++)
	     {
		         int bit0,bit1,bit2;
		         /* red component */
		         bit0 = (dkongpal[c+256] >> 1) & 1;
		         bit1 = (dkongpal[c+256] >> 2) & 1;
		         bit2 = (dkongpal[c+256] >> 3) & 1;
		         yaamepal[c].r = 255 - (0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2);
		         /* green component */
		         bit0 = (dkongpal[c] >> 2) & 1;
               bit1 = (dkongpal[c] >> 3) & 1;
		         bit2 = (dkongpal[c+256] >> 0) & 1;
		         yaamepal[c].g = 255 - (0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2);
		         /* blue component */
		         bit0 = (dkongpal[c] >> 0) & 1;
		         bit1 = (dkongpal[c] >> 1) & 1;
		         yaamepal[c].b = 255 - (0x55 * bit0 + 0xaa * bit1);
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int radarscopeloadgfx()
{
        int c;
        buffer=create_bitmap(512,512);
        clear(buffer);
        read2planeflipped(0,256,"trs2v3hc","trs2v3gc");
        read2planeflipped(256,256,"trs2v3dc","trs2v3cc");
        read2planeflipped(256,256,"trs2v3bc","trs2v3ac");
	     for (c = 0;c < 256;c++)
	     {
		         int bit0,bit1,bit2;
		         /* red component */
		         bit0 = (dkongpal[c+256] >> 1) & 1;
		         bit1 = (dkongpal[c+256] >> 2) & 1;
		         bit2 = (dkongpal[c+256] >> 3) & 1;
		         yaamepal[c].r = 255 - (0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2);
		         /* green component */
		         bit0 = (dkongpal[c] >> 2) & 1;
		         bit1 = (dkongpal[c] >> 3) & 1;
		         bit2 = (dkongpal[c+256] >> 0) & 1;
		         yaamepal[c].g = 255 - (0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2);
		         /* blue component */
		         bit0 = (dkongpal[c] >> 0) & 1;
		         bit1 = (dkongpal[c] >> 1) & 1;
		         yaamepal[c].b = 255 - (0x55 * bit0 + 0xaa * bit1);
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int konginit()
{
        printf("Loading Donkey Kong ROMs\n");
        loadrom("dk.5e",0,4096);
        loadrom("dk.5c",0x1000,4096);
        loadrom("dk.5b",0x2000,4096);
        loadrom("dk.5a",0x3000,4096);
        kongloadgfx();
        initz80(kongread,kongwrite,kongportread,kongportwrite);
        printf("Donkey Kong loaded\n");
        return 0;
}

int radarscopeinit()
{
        printf("Loading Radar Scope ROMs\n");
        loadrom("trs2c5fc",0,4096);
        loadrom("trs2c5gc",0x1000,4096);
        loadrom("trs2c5hc",0x2000,4096);
        loadrom("trs2c5kc",0x3000,4096);
	     ram[0x1E9C]=0xC3;
	     ram[0x1E9D]=0xBD;
        radarscopeloadgfx();
        initz80(kongread,kongwrite,kongportread,kongportwrite);
        printf("Radar Scope loaded\n");
        return 0;
}

void kongdraw()
{
        int c,x,y,pos=0x7400,tile,xflip,colset;
        for (x=0;x<32;x++)
        {
                for (y=0;y<32;y++)
                {
                        colset=(dkongpal[y+512]&0xF)+(palbank<<4);
                        drawtilenew(ram[pos++],(31-x)<<3,y<<3,colset,0,0,2);
                }
        }
        for (c=0;c<80;c++)
        {
                y=ram[0x6900+(c<<2)+3]-8;
                x=ram[0x6900+(c<<2)]-7;
                if (x>0&&y>0)
                {
                        tile=(ram[0x6900+(c<<2)+1]&0x7F)<<1;
                        xflip=ram[0x6900+(c<<2)+1]&0x80;
                        colset=ram[0x6900+(c<<2)+2]&0xF;
                        if (!xflip)
                        {
                                drawtilemasknew(tile+257,x,y,colset,xflip,0,2);
                                drawtilemasknew(tile+256,x+8,y,colset,xflip,0,2);
                                drawtilemasknew(tile+513,x,y+8,colset,xflip,0,2);
                                drawtilemasknew(tile+512,x+8,y+8,colset,xflip,0,2);
                        }
                        else
                        {
                                drawtilemasknew(tile+256,x,y,colset,xflip,0,2);
                                drawtilemasknew(tile+257,x+8,y,colset,xflip,0,2);
                                drawtilemasknew(tile+512,x,y+8,colset,xflip,0,2);
                                drawtilemasknew(tile+513,x+8,y+8,colset,xflip,0,2);
                        }
                }
        }
        waitforsync();
        blit(buffer,screen,0,0,0,0,256,256);
}

void kongrun()
{
        while (!keys[KEY_ESC])
        {
                if (mz80exec(50000)!=0x80000000)
                {
                        closegfx();
                        printf("It crashed\n");
                        exit(-1);
                }
                if (intenable)
                   mz80nmi();
                kongdraw();
        }
}
