/*YAAME - Mr. Do! driver

  CPU   : Z80 (4 mhz)
  Sound : 2xSN76489 (4 mhz)
  Graphics : 192x256, 64 colours

  Known problems :

  Sprite colours bad
*/

#include "gfx.h"
#include <stdio.h>
#include "mem.h"
#include "mz80.h"
#include "yaame.h"
#include "sn.h"

int scrollx;
unsigned char sprites[4096][16][16];

inline int mrdodecode(int val)
{
        int temp=(val&16)?1:0;
        return temp|((val&1)<<1);
}

static void writescrollx(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        scrollx=data;
}

static void mrdowritesound(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        if (addr&1)
           sn76489write(0,data);
        else
           sn76489write(1,data);
}

UINT8 readmrdo(UINT32 addr, struct MemoryReadByte *crap)
{
        mz80GetContext(&z80);
        return ram[z80.z80hl.hl];
}

UINT8 readmrdoin0(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0xFF;
        if (keys[KEY_1])
           temp&=~32;
        if (keys[KEY_UP])
           temp&=~8;
        if (keys[KEY_DOWN])
           temp&=~2;
        if (keys[KEY_LEFT])
           temp&=~1;
        if (keys[KEY_RIGHT])
           temp&=~4;
        if (keys[KEY_SPACE])
           temp&=~16;
        return temp;
}

UINT8 readmrdoin1(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0xFF;
        if (keys[KEY_C])
           temp&=~64;
        return temp;
}

UINT8 readmrdodsw1(UINT32 addr, struct MemoryReadByte *crap)
{
        return 0xFF;
}

UINT8 readmrdodsw2(UINT32 addr, struct MemoryReadByte *crap)
{
        return 0xFF;
}

struct MemoryWriteByte mrdowrite[] =
{
        {0,0x8000,writerom},
        {0x9801,0x9801,snwrite1},
        {0x9802,0x9802,snwrite2},
        {0xF800,0xF800,writescrollx},
        {-1,-1,NULL}
};

struct MemoryReadByte mrdoread[] =
{
        {0xA000,0xA000,readmrdoin0},
        {0xA001,0xA001,readmrdoin1},
        {0xA002,0xA002,readmrdodsw1},
        {0xA003,0xA003,readmrdodsw2},
        {0x9800,0xB000,readmrdo},
        {-1,-1,NULL}
};

struct z80PortRead mrdoportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite mrdoportwrite[] =
{
	{-1,	-1,	NULL}
};

int loadmrdogfx()
{
        FILE *f;
        int y,c;
        unsigned char temp;
        buffer=createbmp(512,512);
        read2plane(0,512,"s8-09.bin","u8-10.bin");
        read2plane(512,512,"r8-08.bin","n8-07.bin");
        f=fopen("h5-05.bin","rb");
        for (c=0;c<64;c++)
        {
                for (y=0;y<16;y++)
                {
                        temp=getc(f);
                        sprites[c][y][15]=mrdodecode(temp);
                        sprites[c][y][14]=mrdodecode(temp>>1);
                        sprites[c][y][13]=mrdodecode(temp>>2);
                        sprites[c][y][12]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c][y][11]=mrdodecode(temp);
                        sprites[c][y][10]=mrdodecode(temp>>1);
                        sprites[c][y][9]=mrdodecode(temp>>2);
                        sprites[c][y][8]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c][y][7]=mrdodecode(temp);
                        sprites[c][y][6]=mrdodecode(temp>>1);
                        sprites[c][y][5]=mrdodecode(temp>>2);
                        sprites[c][y][4]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c][y][3]=mrdodecode(temp);
                        sprites[c][y][2]=mrdodecode(temp>>1);
                        sprites[c][y][1]=mrdodecode(temp>>2);
                        sprites[c][y][0]=mrdodecode(temp>>3);
                }
        }
        fclose(f);
        f=fopen("k5-06.bin","rb");
        for (c=0;c<64;c++)
        {
                for (y=0;y<16;y++)
                {
                        temp=getc(f);
                        sprites[c+64][y][15]=mrdodecode(temp);
                        sprites[c+64][y][14]=mrdodecode(temp>>1);
                        sprites[c+64][y][13]=mrdodecode(temp>>2);
                        sprites[c+64][y][12]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c+64][y][11]=mrdodecode(temp);
                        sprites[c+64][y][10]=mrdodecode(temp>>1);
                        sprites[c+64][y][9]=mrdodecode(temp>>2);
                        sprites[c+64][y][8]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c+64][y][7]=mrdodecode(temp);
                        sprites[c+64][y][6]=mrdodecode(temp>>1);
                        sprites[c+64][y][5]=mrdodecode(temp>>2);
                        sprites[c+64][y][4]=mrdodecode(temp>>3);
                        temp=getc(f);
                        sprites[c+64][y][3]=mrdodecode(temp);
                        sprites[c+64][y][2]=mrdodecode(temp>>1);
                        sprites[c+64][y][1]=mrdodecode(temp>>2);
                        sprites[c+64][y][0]=mrdodecode(temp>>3);
                }
        }
        fclose(f);
        return 0;
}

static unsigned char mrdocolour[] =
{
	/* palette (high bits) */
	0x00,0x0C,0x03,0x00,
	0x0F,0x0B,0x0C,0x3F,
	0x0D,0x0F,0x0F,0x0C,
	0x0C,0x3C,0x0C,0x30,
	0x0C,0x03,0x30,0x03,
	0x0C,0x0F,0x00,0x3F,
	0x03,0x1E,0x00,0x0F,
	0x37,0x36,0x0D,0x33,
	/* palette (low bits) */
	0x00,0x0C,0x03,0x00,
	0x0C,0x03,0x00,0x3F,
	0x0F,0x03,0x0F,0x3F,
	0x0C,0x0F,0x0F,0x3A,
	0x03,0x0F,0x00,0x0C,
	0x00,0x0F,0x3F,0x03,
	0x2A,0x0C,0x00,0x0A,
	0x0C,0x0E,0x3F,0x0F,
	/* sprite color lookup table */
	0x00,0x97,0x71,0xF9,
	0x00,0x27,0xA5,0x13,
	0x00,0x32,0x77,0x3F,
	0x00,0xA7,0x72,0xF9,
	0x00,0x1F,0x9A,0x77,
	0x00,0x15,0x27,0x38,
	0x00,0xC2,0x55,0x69,
	0x00,0x7F,0x76,0x7A
};

void loadpal()
{
        int c,i,j;
	for (i = 0;i < 64;i++)
	{
		for (j = 0;j < 4;j++)
		{
			int bit0,bit1,bit2,bit3;
			bit0=(mrdocolour[4*(i/8)+j+32]>>1)&0x01;
			bit1=(mrdocolour[4*(i/8)+j+32])&0x01;
			bit2=(mrdocolour[4*(i%8)+j]>>1)&0x01;
			bit3=(mrdocolour[4*(i%8)+j])&0x01;
			yaamepal[(4*i + j)].r = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
			bit0=(mrdocolour[4*(i / 8) + j + 32] >> 3) & 0x01;
			bit1=(mrdocolour[4*(i / 8) + j + 32] >> 2) & 0x01;
			bit2=(mrdocolour[4*(i % 8) + j] >> 3) & 0x01;
			bit3=(mrdocolour[4*(i % 8) + j] >> 2) & 0x01;
			yaamepal[(4*i + j)].g = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
			bit0=(mrdocolour[4*(i / 8) + j + 32] >> 5) & 0x01;
			bit1=(mrdocolour[4*(i / 8) + j + 32] >> 4) & 0x01;
			bit2=(mrdocolour[4*(i % 8) + j] >> 5) & 0x01;
			bit3=(mrdocolour[4*(i % 8) + j] >> 4) & 0x01;
			yaamepal[(4*i + j)].b = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
		}
	}
        for (c=0;c<256;c++)
        {
                yaamepal[c].r>>=2;
                yaamepal[c].g>>=2;
                yaamepal[c].b>>=2;
        }
}

int mrdoinit()
{
        FILE *f;
        printf("Loading Mr Do! ROMs\n");
        loadrom("a4-01.bin",0,8192);
        loadrom("c4-02.bin",0x2000,8192);
        loadrom("e4-03.bin",0x4000,8192);
        loadrom("f4-04.bin",0x6000,8192);
        loadmrdogfx();
        loadpal();
        initz80(mrdoread,mrdowrite,mrdoportread,mrdoportwrite);
        sninit(2);
        printf("Mr Do! loaded\n");
        return 0;
}

void mrdodraw()
{
        int y,x;
        int pos,pos2,pos3,pos4;
        unsigned char c;
        int col;
        int xflip,yflip;
        pos=0x8C80;
        pos2=0x8880;
        pos3=0x8480;
        pos4=0x8080;
        for (y = 4; y <28; y++)
        {
                for (x = 31; x >-1; x--)
                {
                      c = ram[pos3];
                      if (ram[pos4]&0x80)
                         drawtilenew(c+768,(y<<3)-scrollx,x<<3,ram[pos4]&0x7F,0,0,2);
                      else
                         drawtilenew(c+512,(y<<3)-scrollx,x<<3,ram[pos4]&0x7F,0,0,2);
                      pos3++;
                      pos4++;
                }
        }
        for (y = 4; y <28; y++)
        {
                for (x = 31; x >-1; x--)
                {
                      c = ram[pos];
                      if (ram[pos2]&0x80)
                      {
                           if (c)
                              drawtilemasknew(c+256,y<<3,x<<3,ram[pos2]&0x7F,0,0,2);
                      }
                      else
                         drawtilemasknew(c,y<<3,x<<3,ram[pos2]&0x7F,0,0,2);
                      pos++;
                      pos2++;
                }
        }
        for (c=0;c<64;c++)
        {
                if (ram[(c<<2)+0x9001]!=0)
                {
                        x=256-ram[(c<<2)+0x9001];
                        y=240-ram[(c<<2)+0x9003];
                        xflip=ram[(c<<2)+0x9002]&0x20;
                        yflip=ram[(c<<2)+0x9002]&0x10;
                        col=ram[(c<<2)+0x9002]&0xF;
                        if (x>0 && y>0)
                           drawspritenew(ram[(c<<2)+0x9000]&127,x,y,col,xflip,yflip,2);
                }
        }
        waitforsync();
        blit(buffer,screen,32,0,0,0,192,256);
}

void mrdorun()
{
        while (!keys[KEY_ESC])
        {
                if (mz80exec(57000)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                mz80int(1);
                mrdodraw();
        }
}
