/*YAAME - Nibbler driver

  CPU : 6502 (800 khz)
  Graphics : 256x256, 16 colours
*/

#include "m6502.h"
#include "gfx.h"
#include <stdio.h>
#include "yaame.h"

unsigned char nibblerctrl;
unsigned char nibblercolour[0x40];

static void nibblerwritectrl(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
      nibblerctrl=data;
}

static void nibblerwritechar(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
      int chr,line,plane;
      line=addr&7;
      chr=(addr>>3)&255;
      plane=(addr&0x800)?1:0;
      bitplane[plane][chr+512][7-line]=data;
      ram[addr]=data;
}

UINT8 nibblerreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0;
        if (keys[KEY_DOWN])
           temp|=0x10;
        if (keys[KEY_UP])
           temp|=0x20;
        if (keys[KEY_RIGHT])
           temp|=0x40;
        if (keys[KEY_LEFT])
           temp|=0x80;
        return temp;
}

UINT8 nibblerreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        return 0;
}

UINT8 nibblerreaddsw(UINT32 addr, struct MemoryReadByte *m)
{
        return 0;
}

UINT8 nibblerreadin2(UINT32 addr, struct MemoryReadByte *m)
{
        if (keys[KEY_1])
           return 0x80;
        return 0;
}

struct MemoryReadByte nibblerread[] =
{
      {0x2104,0x2104,nibblerreadin0},
      {0x2105,0x2105,nibblerreadin1},
      {0x2106,0x2106,nibblerreaddsw},
      {0x2107,0x2107,nibblerreadin2},
      {-1,-1,NULL}
};

struct MemoryWriteByte nibblerwrite[] =
{
      {0x1000,0x1FFF,nibblerwritechar},
      {0x2103,0x2103,nibblerwritectrl},
      {0x3000,0xBFFF,writerom},
      {-1,-1,NULL}
};

int nibblerloadgfx()
{
        FILE *f,*ff;
        int y,c;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        read2planeflipped(0,512,"g960-57.50","g960-58.51");
        f=fopen("g70805.ic7","rb");
        fread(nibblercolour,0x20,1,f);
        fclose(f);
        f=fopen("g70804.ic6","rb");
        fread(nibblercolour+0x20,0x20,1,f);
        fclose(f);
	     for (c=0;c<64;c++)
	     {
		         int bit0,bit1,bit2;
		         bit0 = (nibblercolour[c] >> 0) & 0x01;
		         bit1 = (nibblercolour[c] >> 1) & 0x01;
		         bit2 = (nibblercolour[c] >> 2) & 0x01;
		         yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = (nibblercolour[c] >> 3) & 0x01;
		         bit1 = (nibblercolour[c] >> 4) & 0x01;
		         bit2 = (nibblercolour[c] >> 5) & 0x01;
		         yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		         bit0 = 0;
		         bit1 = (nibblercolour[c] >> 6) & 0x01;
		         bit2 = (nibblercolour[c] >> 7) & 0x01;
		         yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
               yaamepal[c].r/=4;
               yaamepal[c].g/=4;
               yaamepal[c].b/=4;
        }
        return 0;
}

int nibblerinit()
{
        printf("Loading Nibbler ROMs\n");
        loadrom("g960-52.12",0x3000,4096);
        loadrom("g960-48.07",0x4000,4096);
        loadrom("g960-49.08",0x5000,4096);
        loadrom("g960-50.09",0x6000,4096);
        loadrom("g960-51.10",0x7000,4096);
        loadrom("g960-53.14",0x8000,4096);
        loadrom("g960-54.15",0x9000,4096);
        loadrom("g960-55.16",0xA000,4096);
        loadrom("g960-56.17",0xB000,4096);
        loadrom("g960-53.14",0xF000,4096);
        nibblerloadgfx();
        init6502(nibblerread,nibblerwrite);
        printf("Nibbler loaded\n");
        return 0;
}

void nibblerdraw()
{
        int x,y;
        int col;
        int pos=0x800,pos2=0xC00;
        for (x=31;x>-1;x--)
        {
                for (y=0;y<32;y++)
                {
                        col=(ram[pos2++]&0x38)>>3;
                        if (nibblerctrl&8)
                           drawtilenew(ram[pos++],x*8,y*8,col+8,0,0,2);
                        else
                           drawtilenew(ram[pos++]+256,x*8,y*8,col+8,0,0,2);
                }
        }
        pos=0x400;
        pos2=0xC00;
        for (x=31;x>-1;x--)
        {
                for (y=0;y<32;y++)
                {
                        col=ram[pos2++]&7;
                        drawtilemask(ram[pos++]+512,x*8,y*8,col);
                }
        }
        blit(buffer,screen,0,0,0,0,256,256);
}

void nibblerrun()
{
        while (!keys[KEY_ESC])
        {
                if (m6502exec(13333)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                nibblerdraw();
                if (keys[KEY_C])
                   m6502nmi();
                else
                   m6502int(0);
        }
}
