#include "starcpu.h"
#include <stdio.h>
#include "gfx.h"
#include "yaame.h"

volatile int lastfps=0;
volatile int fps=0;

void ticker()
{
      lastfps=fps;
      fps=0;
}

END_OF_FUNCTION(ticker);

unsigned char rom[0x60000];
unsigned char vram[0x10000];
unsigned char spriteram[0x10000];
unsigned char palram[0x10000];
unsigned char tiles[16384][8][8];
unsigned char sprites[4096][16][16];
int firstwritex=1,firstwritey=1;
int p1scrollx,p2scrollx;
int p1scrolly,p2scrolly;
int spritecolbank=0;
FILE *rastanlog;
char ssss[40];

void writerastanscrollx(unsigned address, unsigned data)
{
        if (firstwritex)
        {
               if (data==0)
                  p1scrollx=0;
               else
                  p1scrollx=512-data;
        }
        else
        {
               if (data==0)
                  p2scrollx=0;
               else
                  p2scrollx=512-data;
        }
        firstwritex^=1;
}

void writerastanscrolly(unsigned address, unsigned data)
{
        if (firstwritey)
        {
               if (data==0)
                  p1scrolly=0;
               else
                  p1scrolly=512-data;
        }
        else
        {
               if (data==0)
                  p2scrolly=0;
               else
                  p2scrolly=512-data;
        }
        firstwritey^=1;
}

void writerastanvidctrl(unsigned address, unsigned data)
{
      spritecolbank=(data&0xe0)>>5;
}

unsigned readrastaninput(unsigned address)
{
      unsigned short temp=0xFF;
      sprintf(ssss,"Reading %04X %01X\n",address,address&14);
      fputs(ssss,rastanlog);
      switch (address&14)
      {
            case 0:
            if (keys[KEY_UP])
               temp&=~1;
            if (keys[KEY_DOWN])
               temp&=~2;
            if (keys[KEY_LEFT])
               temp&=~4;
            if (keys[KEY_RIGHT])
               temp&=~8;
            if (keys[KEY_SPACE])
               temp&=~0x10;
            if (keys[KEY_ALT])
               temp&=~0x20;
            return temp;
            case 2:
            return 0xFF;
            case 6:
            temp&=0x1F;
            if (keys[KEY_1])
               temp&=~8;
            if (keys[KEY_C])
               temp|=0x20;
            sprintf(ssss,"%02X\n",temp);
            fputs(ssss,rastanlog);
            return temp;
            case 8:
            return 0x36;
            case 10:
            return 0x60;
            default:
            return 0xFF;
      }
}

struct STARSCREAM_PROGRAMREGION rastanfetch[] =
{
      {0x000000, 0x05FFFF, (unsigned)rom - 0x000000},
      {0x10C000, 0x10FFFF, (unsigned)ram - 0x060000},
      {-1,-1,NULL}
};

struct STARSCREAM_DATAREGION rastanreadbyte[] =
{
      {0x000000, 0x05FFFF, NULL, rom},
      {0x10C000, 0x10FFFF, NULL, ram},
      {0x200000, 0x20FFFF, NULL, palram},
      {0x390000, 0x39000F, readrastaninput, NULL},
      {0xC00000, 0xC0FFFF, NULL, vram},
      {0xD00000, 0xD0FFFF, NULL, spriteram},
      {-1,-1,NULL}
};

struct STARSCREAM_DATAREGION rastanreadword[] =
{
      {0x000000, 0x05FFFF, NULL, rom},
      {0x10C000, 0x10FFFF, NULL, ram},
      {0x200000, 0x20FFFF, NULL, palram},
      {0x390000, 0x39000F, readrastaninput, NULL},
      {0xC00000, 0xC0FFFF, NULL, vram},
      {0xD00000, 0xD0FFFF, NULL, spriteram},
      {-1,-1,NULL}
};

struct STARSCREAM_DATAREGION rastanwritebyte[] =
{
      {0x10C000, 0x10FFFF, NULL, ram},
      {0x200000, 0x20FFFF, NULL, palram},
      {0xC00000, 0xC0FFFF, NULL, vram},
      {0xD00000, 0xD0FFFF, NULL, spriteram},
      {-1,-1,NULL}
};

struct STARSCREAM_DATAREGION rastanwriteword[] =
{
      {0x10C000, 0x10FFFF, NULL, ram},
      {0x200000, 0x20FFFF, NULL, palram},
      {0x380000, 0x380003, writerastanvidctrl, NULL},
      {0xC00000, 0xC0FFFF, NULL, vram},
      {0xC20000, 0xC2FFFF, writerastanscrolly, NULL},
      {0xC40000, 0xC4FFFF, writerastanscrollx, NULL},
      {0xD00000, 0xD0FFFF, NULL, spriteram},
      {-1,-1,NULL}
};

void init68k(struct STARSCREAM_PROGRAMREGION *fetch, struct STARSCREAM_DATAREGION *readbyte, struct STARSCREAM_DATAREGION *readword, struct STARSCREAM_DATAREGION *writebyte, struct STARSCREAM_DATAREGION *writeword)
{
      s68000init();
      s68000context.s_fetch=fetch;
      s68000context.u_fetch=fetch;
      s68000context.s_readbyte=readbyte;
      s68000context.u_readbyte=readbyte;
      s68000context.s_readword=readword;
      s68000context.u_readword=readword;
      s68000context.s_writebyte=writebyte;
      s68000context.u_writebyte=writebyte;
      s68000context.s_writeword=writeword;
      s68000context.u_writeword=writeword;
      s68000SetContext(&s68000context);
      s68000reset();
      printf("Initial PC is %06X\n",s68000context.pc);
}

void rastaninit()
{
        int c,d;
        int y;
        int addr=0;
        unsigned char val;
        FILE *f,*ff;
        memset(ram,0,0x4000);
        memset(vram,0,0x6000);
        printf("Loading program ROMs\n");
        f=fopen("IC19_38.bin","rb");
        ff=fopen("IC07_37.bin","rb");
        for (c=0;c<0x20000;c+=0x2000)
        {
                for (d=0;d<0x2000;d+=2)
                {
                        rom[addr++]=getc(ff);
                        rom[addr++]=getc(f);
                }
                printf(".");
        }
        printf("\n%06X\n",addr);
        fclose(ff);
        fclose(f);
        f=fopen("IC20_40.bin","rb");
        ff=fopen("IC08_39.bin","rb");
        for (c=0;c<0x20000;c+=0x2000)
        {
                for (d=0;d<0x2000;d+=2)
                {
                        rom[addr++]=getc(ff);
                        rom[addr++]=getc(f);
                }
                printf(".");
        }
        printf("\n%06X\n",addr);
        fclose(ff);
        fclose(f);
        f=fopen("IC21_42.bin","rb");
        ff=fopen("IC09_43.bin","rb");
        for (c=0;c<0x20000;c+=0x2000)
        {
                for (d=0;d<0x2000;d+=2)
                {
                        rom[addr++]=getc(ff);
                        rom[addr++]=getc(f);
                }
                printf(".");
        }
        printf("\n");
        fclose(ff);
        fclose(f);

        printf("Loading gfx\n");
        f=fopen("ic40_01.bin","rb");
        ff=fopen("ic67_02.bin","rb");
        for (c=0;c<8192;c+=256)
        {
                for (d=0;d<256;d++)
                {
                        for (y=0;y<8;y++)
                        {
                                val=getc(f);
                                tiles[c+d][0][y]=val>>4;
                                tiles[c+d][1][y]=val&0xF;
                                val=getc(ff);
                                tiles[c+d][2][y]=val>>4;
                                tiles[c+d][3][y]=val&0xF;
                                val=getc(f);
                                tiles[c+d][4][y]=val>>4;
                                tiles[c+d][5][y]=val&0xF;
                                val=getc(ff);
                                tiles[c+d][6][y]=val>>4;
                                tiles[c+d][7][y]=val&0xF;
                        }
                }
                printf(".");
        }
        fclose(ff);
        fclose(f);
        printf("\n");
        f=fopen("ic39_03.bin","rb");
        ff=fopen("ic66_04.bin","rb");
        for (c=0;c<8192;c+=256)
        {
                for (d=0;d<256;d++)
                {
                        for (y=0;y<8;y++)
                        {
                                val=getc(f);
                                tiles[c+d+8192][0][y]=val>>4;
                                tiles[c+d+8192][1][y]=val&0xF;
                                val=getc(ff);
                                tiles[c+d+8192][2][y]=val>>4;
                                tiles[c+d+8192][3][y]=val&0xF;
                                val=getc(f);
                                tiles[c+d+8192][4][y]=val>>4;
                                tiles[c+d+8192][5][y]=val&0xF;
                                val=getc(ff);
                                tiles[c+d+8192][6][y]=val>>4;
                                tiles[c+d+8192][7][y]=val&0xF;
                        }
                }
                printf(".");
        }
        fclose(ff);
        fclose(f);

        printf("\n");

        f=fopen("ic15_05.bin","rb");
        ff=fopen("ic28_06.bin","rb");
        for (c=0;c<2048;c+=256)
        {
                for (d=0;d<256;d++)
                {
                        for (y=0;y<16;y++)
                        {
                                val=getc(f);
                                sprites[c+d][0][y]=val>>4;
                                sprites[c+d][1][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d][2][y]=val>>4;
                                sprites[c+d][3][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d][4][y]=val>>4;
                                sprites[c+d][5][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d][6][y]=val>>4;
                                sprites[c+d][7][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d][8][y]=val>>4;
                                sprites[c+d][9][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d][10][y]=val>>4;
                                sprites[c+d][11][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d][12][y]=val>>4;
                                sprites[c+d][13][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d][14][y]=val>>4;
                                sprites[c+d][15][y]=val&0xF;
                        }
                }
                printf(".");
        }
        fclose(ff);
        fclose(f);
        printf("\n");
        f=fopen("ic14_07.bin","rb");
        ff=fopen("ic27_08.bin","rb");
        for (c=0;c<2048;c+=256)
        {
                for (d=0;d<256;d++)
                {
                        for (y=0;y<16;y++)
                        {
                                val=getc(f);
                                sprites[c+d+2048][0][y]=val>>4;
                                sprites[c+d+2048][1][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d+2048][2][y]=val>>4;
                                sprites[c+d+2048][3][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d+2048][4][y]=val>>4;
                                sprites[c+d+2048][5][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d+2048][6][y]=val>>4;
                                sprites[c+d+2048][7][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d+2048][8][y]=val>>4;
                                sprites[c+d+2048][9][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d+2048][10][y]=val>>4;
                                sprites[c+d+2048][11][y]=val&0xF;
                                val=getc(f);
                                sprites[c+d+2048][12][y]=val>>4;
                                sprites[c+d+2048][13][y]=val&0xF;
                                val=getc(ff);
                                sprites[c+d+2048][14][y]=val>>4;
                                sprites[c+d+2048][15][y]=val&0xF;
                        }
                }
                printf(".");
        }
        fclose(ff);
        fclose(f);

        printf("\n");
        init68k(rastanfetch,rastanreadbyte,rastanreadword,rastanwritebyte,rastanwriteword);
        buffer=createbmp(640,640);
        printf("Lets execute the code\n");
}

void printcpureg()
{
	int i;
	printf(
		"d0=%08X   d4=%08X   a0=%08X   a4=%08X   %c%c%c%c%c\n",
		s68000context.dreg[0],s68000context.dreg[4],
		s68000context.areg[0],s68000context.areg[4],
		(s68000context.sr >> 4) & 1 ? 'X' : '-',
		(s68000context.sr >> 3) & 1 ? 'N' : '-',
		(s68000context.sr >> 2) & 1 ? 'Z' : '-',
		(s68000context.sr >> 1) & 1 ? 'V' : '-',
		(s68000context.sr     ) & 1 ? 'C' : '-'
	);
	printf(
		"d1=%08X   d5=%08X   a1=%08X   a5=%08X   intmask=%d\n",
		s68000context.dreg[1], s68000context.dreg[5],
		s68000context.areg[1], s68000context.areg[5],
		(s68000context.sr >> 8) & 7
	);
	printf(
		"d2=%08X   d6=%08X   a2=%08X   a6=%08X   ",
		s68000context.dreg[2], s68000context.dreg[6],
		s68000context.areg[2], s68000context.areg[6]
	);
	for(i = 1; i <= 7; i++) {
		if(s68000context.interrupts[0] & (1 << i)) {
			printf("%02X", s68000context.interrupts[i]);
		} else {
			printf("--");
		}
	}
	printf(
		"\nd3=%08X   d7=%08X   a3=%08X   a7=%08X   %csp=%08X\n",
		s68000context.dreg[3], s68000context.dreg[7],
		s68000context.areg[3], s68000context.areg[7],
		((s68000context.sr)&0x2000)?'u':'s', s68000context.asp
	);
        printf("PC was %06X\n",s68000context.pc);
}

inline void drawtilenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

inline void drawtilemasknew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

inline void drawspritenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
      int xx,yy;
      if (!yflip)
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
      else
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
}

/*Oldskool drawing code
                for (x=0;x<40;x++)
                {
                        if ((((y*8)-p1scrolly)&511)<255)
                        {
                                dat1=vram[addr]|(vram[addr+1]<<8);
                                dat2=vram[addr+2]|(vram[addr+3]<<8);
                                drawtilenew(dat2&0x3FFF,x*8,((unsigned)((y*8)-p1scrolly))&255,dat1&0xF);
                        }
                        addr+=4;
                        if ((((y*8)-p2scrolly)&511)<255)
                        {
                                dat1=vram[addr2]|(vram[addr2+1]<<8);
                                dat2=vram[addr2+2]|(vram[addr2+3]<<8);
                                drawtilemasknew(dat2&0x3FFF,x*8,((unsigned)((y*8)-p2scrolly))&255,dat1&0xF);
                        }
                        addr2+=4;
                }
*/
void letsdrawthescreen()
{
        int x,y,addr=0x0000,addr2=0x8000;
        unsigned short dat1,dat2;
        int tile,col;
        int c;
        for (y=0;y<64;y++)
        {
                if ((((y<<3)-p1scrolly)&511)<255)
                {
                        for (x=0;x<64;x++)
                        {
                                if ((((x<<3)-p1scrollx)&511)<335)
                                {
                                       dat2=vram[addr+2]|(vram[addr+3]<<8);
                                       dat1=vram[addr]|(vram[addr+1]<<8);
                                       drawtilenew(dat2&0x3FFF,((x<<3)-p1scrollx)&511,((unsigned)((y*8)-p1scrolly))&255,dat1&0xF,0,0,4);
                                }
                                addr+=4;
                        }
//                        addr+=96;
                }
                else
                   addr+=256;
                if ((((y<<3)-p2scrolly)&511)<255)
                {
                        for (x=0;x<64;x++)
                        {
                                if ((((x<<3)-p2scrollx)&511)<335)
                                {
                                       dat2=vram[addr2+2]|(vram[addr2+3]<<8);
                                       if ((dat2&0x3FFF)!=0)
                                       {
                                             dat1=vram[addr2]|(vram[addr2+1]<<8);
                                             drawtilemasknew(dat2&0x3FFF,((x<<3)-p2scrollx)&511,((unsigned)((y<<3)-p1scrolly))&255,dat1&0xF,0,0,4);
                                       }
                                }
                                addr2+=4;
                        }
//                        addr2+=96;
                }
                else
                   addr2+=256;
        }
        addr=0x7F8;
        for (c=0;c<256;c++)
        {
               tile=spriteram[addr+4]|(spriteram[addr+5]<<8);
               if (tile)
               {
                     x=spriteram[addr+6]|(spriteram[addr+7]<<8);
                     x&=511;
                     y=spriteram[addr+2]|(spriteram[addr+3]<<8);
                     y&=511;
                     col=spriteram[addr]&0xF|(spritecolbank<<4);
                     drawspritenew(tile&0xFFF,x+8,y,col,spriteram[addr+1]&0x40,spriteram[addr+1]&0x80,4);
               }
               addr-=8;
        }
        addr=0;
        for (c=0;c<256;c++)
        {
                yaamepal[c].r=(palram[addr]&31)<<1;
                dat1=palram[addr]>>5;
                dat2=palram[addr+1]&3;
                yaamepal[c].g=(dat1|(dat2<<3))<<1;
                yaamepal[c].b=((palram[addr+1]>>2)&31)<<1;
                addr+=2;
        }
        textprintf(buffer,font,0,248,makecol(255,255,255),"%i %i %i %i %i %i",p1scrollx,p1scrolly,p2scrollx,p2scrolly,lastfps,spritecolbank);
        blit(buffer,screen,0,0,0,0,336,256);
        setpal(yaamepal);
        fps++;
}

void letsdumptheram()
{
        FILE *f;
        f=fopen("rastan.dmp","wb");
        fwrite(ram,0x4000,1,f);
        fclose(f);
        f=fopen("rastanv.dmp","wb");
        fwrite(vram,0x10000,1,f);
        fclose(f);
}

void rastanrun()
{
      unsigned long result;
      rastanlog=fopen("rastan.log","wt");
      LOCK_VARIABLE(lastfps);
      LOCK_VARIABLE(fps);
      LOCK_FUNCTION(ticker);
      install_int_ex(ticker,BPS_TO_TIMER(1));
      while (!keys[KEY_ESC])
      {
            result=s68000exec(133333);
            if (result!=0x80000000)
            {
                  closegfx();
                  printcpureg();
                  printf("Bad result : %08X\n",result);
                  exit(-1);
            }
            s68000interrupt(5,-1);
            result=s68000exec(133333);
            if (result!=0x80000000)
            {
                  closegfx();
                  printcpureg();
                  printf("Bad result : %08X\n",result);
                  exit(-1);
            }
            s68000interrupt(5,-1);
            letsdrawthescreen();
      }
      letsdumptheram();
}
