/*YAAME - Space Invaders driver
  Also supports Space Invaders II and Earth Invasion

  CPU : i8080 (2 mhz)
  Graphics : 224x256, monochrome
*/

#include <stdio.h>
#include "gfx.h"
#include "mem.h"
#include "mz80.h"
#include "yaame.h"

unsigned char shiftdat1,shiftdat2,shiftamount;
unsigned char sichange[0x4000];

/*Read player controls*/
UINT16 sireadplayer(UINT16 port, struct z80PortRead *pPR)
{
        unsigned char temp=0x81;
        switch (port)
        {
                case 0:
                return 0xF4;
                case 1:
                if (keys[KEY_C])
                   temp&=0xFE;
                if (keys[KEY_1])
                   temp|=4;
                if (keys[KEY_LEFT])
                   temp|=0x20;
                if (keys[KEY_RIGHT])
                   temp|=0x40;
                if (keys[KEY_SPACE])
                   temp|=0x10;
                return (UINT16)temp;
                case 2:
                return 0;
        }
        return 0xF4;
}

UINT16 sireadshift(UINT16 port, struct z80PortRead *pPR)
{
	     return ((((shiftdat1 << 8) | shiftdat2) << shiftamount) >> 8) & 0xFF;
}

static void siwriteamount(UINT16 port, UINT8 data, struct z80PortWrite *pPW)
{
        shiftamount=data&7;
}

static void siwriteshift(UINT16 port, UINT8 data, struct z80PortWrite *pPW)
{
        shiftdat2=shiftdat1;
        shiftdat1=data;
}

void writesivid(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        sichange[addr]=1;
        ram[addr]=data;
}

struct z80PortRead siportread[] =
{
        {0x0000,0x0002,sireadplayer,NULL},
        {0x0003,0x0003,sireadshift,NULL},
	     {-1,-1,NULL}
};

struct z80PortWrite siportwrite[] =
{
	     {0x02,0x02,siwriteamount},
        {0x04,0x04,siwriteshift},
	     {-1,-1,NULL}
};

struct MemoryWriteByte siwritemem[] =
{
	     {0x0000,0x1fff,writerom},
        {0x2400,0x4000,writesivid},
	     {-1,-1,NULL}
};

struct MemoryReadByte sireadmem[] =
{
	     {-1, -1, NULL}
};

int siinit()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading Space Invaders\n");
        loadrom("invaders.h",0,2048);
        loadrom("invaders.g",0x800,2048);
        loadrom("invaders.f",0x1000,2048);
        loadrom("invaders.e",0x1800,2048);
        init8080(sireadmem,siwritemem,siportread,siportwrite);
        yaamepal[0].r=yaamepal[0].g=yaamepal[0].b=0;
        yaamepal[1].r=yaamepal[1].g=yaamepal[1].b=63;
        printf("Space Invaders loaded\n");
}

int si2init()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading Space Invaders II\n");
        loadrom("invdelux.h",0,2048);
        loadrom("invdelux.g",0x800,2048);
        loadrom("invdelux.f",0x1000,2048);
        loadrom("invdelux.e",0x1800,2048);
        loadrom("invdelux.d",0x4000,2048);
        init8080(sireadmem,siwritemem,siportread,siportwrite);
        yaamepal[0].r=yaamepal[0].g=yaamepal[0].b=0;
        yaamepal[1].r=yaamepal[1].g=yaamepal[1].b=63;
        printf("Space Invaders II loaded\n");
}

int eiinit()
{
        FILE *f;
        buffer=createbmp(256,256);
        printf("Loading Earth Invasion\n");
        loadrom("earthinv.h",0,2048);
        loadrom("earthinv.g",0x800,2048);
        loadrom("earthinv.f",0x1000,2048);
        loadrom("earthinv.e",0x1800,2048);
        init8080(sireadmem,siwritemem,siportread,siportwrite);
        yaamepal[0].r=yaamepal[0].g=yaamepal[0].b=0;
        yaamepal[1].r=yaamepal[1].g=yaamepal[1].b=63;
        printf("Earth Invasion loaded\n");
}

void sidraw()
{
        int x,y,yy,addr=0x2400;
        unsigned char byte;
        for (addr=0x2400;addr<0x4000;addr++)
        {
                byte=ram[addr];
                if (sichange[addr])
                {
                        sichange[addr]=0;
	                     x=(addr-0x2400)>>5;
	                     y=(addr-0x2400)&0x1F;
                        y<<=3;
                        for (yy=0;yy<8;yy++)
                        {
                                buffer->line[255-(y+yy)][x]=byte&1;
                                byte>>=1;
                        }
                }
        }
        waitforsync();
        blit(buffer,screen,0,0,0,0,224,256);
}

int screencount=0;
int intcount=0;
int intstatus=0;

void sirun()
{
        memset(sichange,1,0x4000);
        while (!keys[KEY_ESC])
        {
                if (mz80exec(17067)!=0x80000000)
                {
                        closegfx();
                        printf("It crashed\n");
                        exit(-1);
                }
                screencount+=17067;
                if (screencount>33333)
                {
                        sidraw();
                        screencount-=33333;
                }
                if (intstatus)
                   mz80nmi();
                else
                   mz80int(0);
                intstatus^=1;
        }
}
