#ifdef _MZ80_H_
/*The z80 context*/
struct mz80context z80;
#endif

#ifdef _M6502_H_
/*The 6502 context*/
struct m6502context m6502;
#endif

/*Graphics structures*/
unsigned char bitplane[4][4096][8];
unsigned char tiles[16384][8][8];
unsigned char sprites[4096][16][16];

/*The emulated memory*/
unsigned char ram[65536];

unsigned char nmi;
/*Graphics palette and back buffer*/
PALETTE yaamepal;
BMP *buffer;

void inityaame();
void waitforsync();

void initz80(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite);
void init8080(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite);
void init6502(struct MemoryReadByte *read, struct MemoryWriteByte *write);

/*Common memory write functions*/
void writerom(unsigned long addr, unsigned char data, struct MemoryWriteByte *pMemWrite);
void nmienable(unsigned long addr, unsigned char data, struct MemoryWriteByte *pMemWrite);

/*Draws a tile onto the back buffer from the bitplane structure*/
extern inline void drawtile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draws a tile onto the back buffer from the bitplane structure with masking*/
extern inline void drawtilemask(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draw a Galaxian style sprite onto the back buffer with masking*/
extern inline void drawsprite(int tile, unsigned char x, unsigned int y, unsigned char colset, int xflip, int yflip)
{
        int xx,yy;
        unsigned char byte1,byte2;
        if (!xflip)
        {
                for (yy=0;yy<8;yy++)
                {
                        byte2=bitplane[0][tile][yy];
                        byte1=bitplane[1][tile][yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
        else
        {
                for (yy=7;yy>-1;yy--)
                {
                        byte2=bitplane[0][tile][7-yy];
                        byte1=bitplane[1][tile][7-yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
}

extern inline void drawtile2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

extern inline void drawtilemask2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

extern inline void drawtilenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

extern inline void drawtilemasknew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
        int xx,yy;
        if (!yflip)
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][yy]|(col<<bits);
               }
        }
        else
        {
               if (!xflip)
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][xx][7-yy]|(col<<bits);
               }
               else
               {
                        for (yy=0;yy<8;yy++)
                            for (xx=0;xx<8;xx++)
                                if (tiles[tile&0x3FFF][7-xx][7-yy])
                                   buffer->line[y+yy][x+xx]=tiles[tile&0x3FFF][7-xx][7-yy]|(col<<bits);
               }
        }
}

extern inline void drawspritenew(int tile, int x, int y, int col, int xflip, int yflip, int bits)
{
      int xx,yy;
      if (!yflip)
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[yy+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
      else
      {
            if (!xflip)
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][xx+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
            else
            {
                  for (yy=0;yy<16;yy++)
                  {
                        for (xx=0;xx<16;xx++)
                        {
                              if (sprites[tile][xx][yy])
                                 buffer->line[(15-yy)+y][(15-xx)+x]=sprites[tile][xx][yy]|(col<<bits);
                        }
                  }
            }
      }
}
